#!/usr/bin/env python3
"""
Handles processing of order fills and disappeared orders.
"""

import logging
import asyncio
from datetime import datetime, timedelta, timezone
from typing import Optional, Dict, Any, List

from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class OrderFillProcessor:
    def __init__(self, trading_engine, notification_manager, market_monitor_cache):
        self.trading_engine = trading_engine
        self.notification_manager = notification_manager
        self.market_monitor_cache = market_monitor_cache # To access cached orders/positions
        # Add necessary initializations

    # Methods like _check_order_fills, _process_disappeared_orders, _activate_pending_stop_losses_from_trades will go here
    pass 

    async def _check_order_fills(self):
        """Check for filled orders and send notifications."""
        try:
            # Get current orders and positions
            current_orders = self.market_monitor_cache.cached_orders or [] # Use cache
            # current_positions = self.market_monitor_cache.cached_positions or [] # Use cache. Not directly used here.
            
            # Get current order IDs
            current_order_ids = {order.get('id') for order in current_orders if order.get('id')}
            
            # Find filled orders (orders that were in last_known_orders but not in current_orders)
            disappeared_order_ids = self.market_monitor_cache.last_known_orders - current_order_ids
            
            if disappeared_order_ids:
                logger.info(f"🎯 Detected {len(disappeared_order_ids)} bot orders no longer open: {list(disappeared_order_ids)}. Corresponding fills (if any) are processed by external trade checker.")
                await self._process_disappeared_orders(disappeared_order_ids)
            
            # Update tracking data for open bot orders
            self.market_monitor_cache.last_known_orders = current_order_ids
            
        except Exception as e:
            logger.error(f"❌ Error checking order fills: {e}")

    async def _process_disappeared_orders(self, disappeared_order_ids: set):
        """Log and investigate bot orders that have disappeared from the exchange."""
        stats = self.trading_engine.get_stats()
        if not stats:
            logger.warning("⚠️ TradingStats not available in _process_disappeared_orders.")
            return

        try:
            total_linked_cancelled = 0
            external_cancellations = []
            
            for exchange_oid in disappeared_order_ids:
                order_in_db = stats.get_order_by_exchange_id(exchange_oid)
                
                if order_in_db:
                    last_status = order_in_db.get('status', 'unknown')
                    order_type = order_in_db.get('type', 'unknown')
                    symbol = order_in_db.get('symbol', 'unknown')
                    token = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]
                    
                    logger.info(f"Order {exchange_oid} was in our DB with status '{last_status}' but has now disappeared from exchange.")
                    
                    active_statuses = ['open', 'submitted', 'partially_filled', 'pending_submission']
                    if last_status in active_statuses:
                        # Before declaring external cancellation, check for very recent fills for this OID
                        # This is a quick check to see if a fill came through just before it disappeared
                        # and _check_external_trades hasn't processed it yet.
                        # This check is specific to this "disappeared orders" context.
                        # fill_just_processed = await self._check_for_recent_fills_for_order(exchange_oid, order_in_db)
                        # if fill_just_processed:
                        #     logger.info(f"ℹ️ Order {exchange_oid} disappeared, but a recent fill was found. Assuming filled. Main fill processing will handle lifecycle.")
                        #     # Potentially update order_in_db status here or rely on main fill processor
                        #     continue # Skip to next disappeared_order_id

                        # If no immediate fill found by the helper, proceed with external cancellation logic
                        logger.warning(f"⚠️ EXTERNAL CANCELLATION: Order {exchange_oid} with status '{last_status}' was likely cancelled externally on Hyperliquid")
                        stats.update_order_status(exchange_order_id=exchange_oid, new_status='cancelled_externally')
                        
                        external_cancellations.append({
                            'exchange_oid': exchange_oid,
                            'token': token,
                            'type': order_type,
                            'last_status': last_status
                        })
                        
                        if self.notification_manager:
                            await self.notification_manager.send_generic_notification(
                                f"⚠️ <b>External Order Cancellation Detected</b>\n\n"
                                f"Token: {token}\n"
                                f"Order Type: {order_type.replace('_', ' ').title()}\n"
                                f"Exchange Order ID: <code>{exchange_oid[:8]}...</code>\n"
                                f"Previous Status: {last_status.replace('_', ' ').title()}\n"
                                f"Source: Cancelled directly on Hyperliquid\n"
                                f"Time: {datetime.now(timezone.utc).strftime('%H:%M:%S')}\n\n"
                                f"🤖 Bot status updated automatically"
                            )
                            
                        order_after_external_check = stats.get_order_by_exchange_id(exchange_oid)
                        if order_after_external_check and order_after_external_check.get('status') == 'cancelled_externally':
                            pending_lc = stats.get_lifecycle_by_entry_order_id(exchange_oid, status='pending')
                            if pending_lc:
                                lc_id_to_cancel = pending_lc.get('trade_lifecycle_id')
                                if lc_id_to_cancel:
                                    cancel_reason = f"entry_order_{exchange_oid[:8]}_disappeared_externally"
                                    cancelled_lc_success = stats.update_trade_cancelled(lc_id_to_cancel, reason=cancel_reason)
                                    if cancelled_lc_success:
                                        logger.info(f"🔗 Trade lifecycle {lc_id_to_cancel} also cancelled for disappeared entry order {exchange_oid}.")
                                        if self.notification_manager:
                                            await self.notification_manager.send_generic_notification(
                                                f"�� <b>Trade Lifecycle Cancelled</b>\n\n"
                                                f"Token: {token}\n"
                                                f"Lifecycle ID: {lc_id_to_cancel[:8]}...\n"
                                                f"Reason: Entry order {exchange_oid[:8]}... cancelled externally.\n"
                                                f"Time: {datetime.now(timezone.utc).strftime('%H:%M:%S')}"
                                            )
                                    else:
                                        logger.error(f"❌ Failed to cancel trade lifecycle {lc_id_to_cancel} for entry order {exchange_oid}.")

                        elif order_after_external_check and order_after_external_check.get('status') in ['filled', 'partially_filled']:
                            logger.info(f"ℹ️ Order {exchange_oid} was ultimately found to be '{order_after_external_check.get('status')}' despite initial disappearance. Stop losses will not be cancelled by this path. Lifecycle should be active.")
                            continue

                    else:
                        if last_status not in ['filled', 'partially_filled', 'cancelled_manually', 'cancelled_by_bot', 'failed_submission', 'cancelled_externally']:
                            stats.update_order_status(exchange_order_id=exchange_oid, new_status='disappeared_from_exchange')
                        
                    if order_in_db.get('bot_order_ref_id'):
                        parent_order_current_state = stats.get_order_by_exchange_id(exchange_oid)
                        if parent_order_current_state and parent_order_current_state.get('status') not in ['filled', 'partially_filled']:
                            logger.info(f"Cancelling stop losses for order {exchange_oid} (status: {parent_order_current_state.get('status')}) as it is not considered filled.")
                            cancelled_sl_count = stats.cancel_linked_orders(
                                parent_bot_order_ref_id=order_in_db['bot_order_ref_id'],
                                new_status='cancelled_parent_disappeared_or_not_filled'
                            )
                            total_linked_cancelled += cancelled_sl_count
                        
                            if cancelled_sl_count > 0:
                                logger.info(f"Cancelled {cancelled_sl_count} pending stop losses linked to disappeared/non-filled order {exchange_oid}")
                                if self.notification_manager:
                                    await self.notification_manager.send_generic_notification(
                                        f"🛑 <b>Linked Stop Losses Cancelled</b>\n\n"
                                        f"Token: {token}\n"
                                        f"Cancelled: {cancelled_sl_count} stop loss(es)\n"
                                        f"Reason: Parent order {exchange_oid[:8]}... disappeared or was not filled\n"
                                        f"Parent Status: {parent_order_current_state.get('status', 'N/A').replace('_', ' ').title()}\n"
                                        f"Time: {datetime.now(timezone.utc).strftime('%H:%M:%S')}"
                                    )
                        else:
                            logger.info(f"ℹ️ Stop losses for order {exchange_oid} (status: {parent_order_current_state.get('status') if parent_order_current_state else 'N/A'}) preserved as parent order is considered filled or partially filled.")
                            continue
                else:
                    logger.warning(f"Order {exchange_oid} disappeared from exchange but was not found in our DB. This might be an order placed externally.")

            if len(external_cancellations) > 1:
                tokens_affected = list(set(item['token'] for item in external_cancellations))
                if self.notification_manager:
                    await self.notification_manager.send_generic_notification(
                        f"⚠️ <b>Multiple External Cancellations Detected</b>\n\n"
                        f"Orders Cancelled: {len(external_cancellations)}\n"
                        f"Tokens Affected: {', '.join(tokens_affected)}\n"
                        f"Source: Direct cancellation on Hyperliquid\n"
                        f"Linked Stop Losses Cancelled: {total_linked_cancelled}\n"
                        f"Time: {datetime.now(timezone.utc).strftime('%H:%M:%S')}\n\n"
                        f"💡 Check individual orders for details"
                    )
        except Exception as e:
            logger.error(f"❌ Error processing disappeared orders: {e}", exc_info=True)

    async def _check_for_recent_fills_for_order(self, exchange_oid, order_in_db):
        """Check for very recent fills that might match this order."""
        # This method checks for fills that might have occurred *just before* an order disappeared,
        # but *before* the main _check_external_trades might have run for the current cycle.
        # It uses its own tracking of last_processed_trade_time or a default if not available.
        try:
            recent_fills = self.trading_engine.get_recent_fills()
            if not recent_fills:
                return False

            # This last_processed_trade_time is for the context of this specific helper,
            # to avoid re-checking fills that the broader external trade monitor might have already seen.
            # It attempts to use the global one if available.
            # The key 'last_processed_trade_time' might be distinct from 'market_monitor_last_processed_trade_time'.
            # For safety, let's ensure this is consistently named if it's meant to be the same.
            # Given it's a helper within OrderFillProcessor, and external trades are separate,
            # we will keep its independent loading logic for now.
            # If MarketMonitor centralizes this timestamp, this should be updated.
            
            # Attempt to load the specific last_processed_trade_time if not already set on this instance
            # This implies self.last_processed_trade_time is an attribute of OrderFillProcessor
            if not hasattr(self, 'last_processed_trade_time_helper') or self.last_processed_trade_time_helper is None:
                try:
                    # Using a distinct metadata key for this helper to avoid conflict,
                    # or assuming it should use the global one. For now, let's assume it tries to use the global.
                    last_time_str = self.trading_engine.stats._get_metadata('market_monitor_last_processed_trade_time')
                    if last_time_str:
                        self.last_processed_trade_time_helper = datetime.fromisoformat(last_time_str).replace(tzinfo=timezone.utc)
                    else:
                        self.last_processed_trade_time_helper = datetime.now(timezone.utc) - timedelta(hours=1)
                except Exception: 
                     self.last_processed_trade_time_helper = datetime.now(timezone.utc) - timedelta(hours=1)


            for fill in recent_fills:
                try:
                    trade_id = fill.get('id')
                    timestamp_ms = fill.get('timestamp')
                    symbol_from_fill = fill.get('symbol')
                    side_from_fill = fill.get('side')
                    amount_from_fill = float(fill.get('amount', 0))
                    price_from_fill = float(fill.get('price', 0))
                    
                    timestamp_dt = datetime.fromtimestamp(timestamp_ms / 1000, tz=timezone.utc) if timestamp_ms else datetime.now(timezone.utc)
                    
                    if timestamp_dt <= self.last_processed_trade_time_helper:
                        continue
                    
                    if symbol_from_fill and side_from_fill and amount_from_fill > 0 and price_from_fill > 0:
                        exchange_order_id_from_fill = fill.get('info', {}).get('oid')
                        
                        if exchange_order_id_from_fill == exchange_oid:
                            if order_in_db.get('symbol') == symbol_from_fill and \
                               order_in_db.get('side') == side_from_fill and \
                               abs(float(order_in_db.get('amount_requested', 0)) - amount_from_fill) < 0.01 * amount_from_fill :
                                logger.info(f"✅ Found recent matching fill {trade_id} for order {exchange_oid}. Not cancelling stop losses.")
                                # This fill should be processed by the main external trade checker.
                                # For the purpose of this helper, just confirming a fill exists is enough.
                                return True
                
                except Exception as e:
                    logger.error(f"Error processing fill {fill.get('id','N/A')} in _check_for_recent_fills_for_order: {e}")
                    continue
            
            return False

        except Exception as e:
            logger.error(f"❌ Error in _check_for_recent_fills_for_order for OID {exchange_oid}: {e}", exc_info=True)
            return False 

    async def _activate_pending_stop_losses_from_trades(self):
        """🆕 PHASE 4: Check trades table for pending stop loss activation first (highest priority)"""
        try:
            stats = self.trading_engine.get_stats()
            if not stats:
                return
            
            formatter = get_formatter() 
            trades_needing_sl = stats.get_pending_stop_loss_activations()
            
            if not trades_needing_sl:
                return
            
            logger.debug(f"🆕 Found {len(trades_needing_sl)} open positions needing stop loss activation")
            
            for position_trade in trades_needing_sl:
                try:
                    symbol = position_trade['symbol']
                    token = symbol.split('/')[0] if symbol and '/' in symbol else (symbol if symbol else "TOKEN")
                    stop_loss_price = position_trade['stop_loss_price']
                    position_side = position_trade['position_side'] 
                    current_amount = position_trade.get('current_position_size', 0)
                    lifecycle_id = position_trade['trade_lifecycle_id']

                    if not all([symbol, stop_loss_price, position_side, abs(current_amount) > 1e-9, lifecycle_id]):
                        logger.warning(f"Skipping SL activation for lifecycle {lifecycle_id} due to incomplete data: sym={symbol}, sl_price={stop_loss_price}, side={position_side}, amt={current_amount}")
                        continue
                    
                    logger.info(f"Attempting to place LIMIT stop loss for lifecycle {lifecycle_id} ({position_side} {token} @ SL {formatter.format_price(stop_loss_price, symbol)})")
                    sl_result = await self.trading_engine.place_limit_stop_for_lifecycle(
                        lifecycle_id=lifecycle_id,
                        symbol=symbol,
                        sl_price=stop_loss_price,
                        position_side=position_side,
                        amount_to_cover=abs(current_amount) 
                    )
                        
                    if sl_result.get('success'):
                        placed_sl_details = sl_result.get('order_placed_details', {})
                        sl_exchange_order_id = placed_sl_details.get('exchange_order_id')
                        sl_db_order_id = placed_sl_details.get('order_db_id')
                        stop_loss_price_str_log = formatter.format_price_with_symbol(stop_loss_price, token)

                        logger.info(f"✅ Successfully processed SL request for {token} (Lifecycle: {lifecycle_id[:8]}): SL Price {stop_loss_price_str_log}, Exchange SL Order ID: {sl_exchange_order_id or 'N/A'}, DB ID: {sl_db_order_id or 'N/A'}")
                        
                        if self.notification_manager and sl_exchange_order_id:
                            current_price_for_notification = None
                            try:
                                market_data_notify = self.trading_engine.get_market_data(symbol)
                                if market_data_notify and market_data_notify.get('ticker'):
                                    current_price_for_notification = float(market_data_notify['ticker'].get('last', 0))
                            except:
                                pass 

                            current_price_str_notify = formatter.format_price_with_symbol(current_price_for_notification, token) if current_price_for_notification else 'Unknown'
                            stop_loss_price_str_notify = formatter.format_price_with_symbol(stop_loss_price, token)
                            
                            await self.notification_manager.send_generic_notification(
                                f"🛡️ <b>Stop Loss LIMIT Order Placed</b>\n\n"
                                f"Token: {token}\n"
                                f"Lifecycle ID: {lifecycle_id[:8]}...\n"
                                f"Position Type: {position_side.upper()}\n"
                                f"Stop Loss Price: {stop_loss_price_str_notify}\n"
                                f"Amount: {formatter.format_amount(abs(current_amount), token)}\n"
                                f"Current Price: {current_price_str_notify}\n"
                                f"Exchange SL Order ID: {sl_exchange_order_id}\n"
                                f"Time: {datetime.now(timezone.utc).strftime('%H:%M:%S')}"
                            )
                        elif not sl_exchange_order_id:
                             logger.warning(f"SL Limit order for {token} (Lifecycle: {lifecycle_id[:8]}) placed in DB (ID: {sl_db_order_id}) but no exchange ID returned immediately.")

                    else:
                        logger.error(f"❌ Failed to place SL limit order for {token} (Lifecycle: {lifecycle_id[:8]}): {sl_result.get('error')}")

                except Exception as trade_error:
                    logger.error(f"❌ Error processing position trade for SL activation (Lifecycle: {position_trade.get('trade_lifecycle_id','N/A')}): {trade_error}")
            
        except Exception as e:
            logger.error(f"❌ Error activating pending stop losses from trades table: {e}", exc_info=True) 