#!/usr/bin/env python3
"""
Performance Calculator for Trading Statistics

Handles performance metrics calculations including win rate, PnL, drawdown,
trade durations, and comprehensive statistical analysis.
"""

import logging
from datetime import datetime, timezone, timedelta
from typing import Dict, List, Any, Optional, Tuple
import math
import numpy as np
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class PerformanceCalculator:
    """Calculates performance metrics and statistics from trading data."""

    def __init__(self, db_manager):
        """Initialize with database manager."""
        self.db = db_manager

    def _format_duration(self, total_seconds: int) -> str:
        """Format duration from seconds to human-readable format."""
        if total_seconds < 60:
            return f"{int(total_seconds)}s"
        elif total_seconds < 3600:
            minutes = total_seconds // 60
            seconds = total_seconds % 60
            return f"{int(minutes)}m {int(seconds)}s"
        elif total_seconds < 86400:
            hours = total_seconds // 3600
            minutes = (total_seconds % 3600) // 60
            return f"{int(hours)}h {int(minutes)}m"
        else:
            days = total_seconds // 86400
            hours = (total_seconds % 86400) // 3600
            return f"{int(days)}d {int(hours)}h"

    def get_performance_stats(self) -> Dict[str, Any]:
        """Get comprehensive trading performance statistics."""
        # Get token-level stats
        token_stats = self.db._fetch_query("SELECT * FROM token_stats ORDER BY total_realized_pnl DESC")
        
        # Calculate overall aggregated metrics
        total_realized_pnl = sum(t.get('total_realized_pnl', 0) for t in token_stats)
        total_completed_cycles = sum(t.get('total_completed_cycles', 0) for t in token_stats)
        total_winning_cycles = sum(t.get('winning_cycles', 0) for t in token_stats)
        total_losing_cycles = sum(t.get('losing_cycles', 0) for t in token_stats)
        total_cancelled_cycles = sum(t.get('total_cancelled_cycles', 0) for t in token_stats)
        total_entry_volume = sum(t.get('total_entry_volume', 0) for t in token_stats)
        total_exit_volume = sum(t.get('total_exit_volume', 0) for t in token_stats)
        sum_of_winning_pnl = sum(t.get('sum_of_winning_pnl', 0) for t in token_stats)
        sum_of_losing_pnl = sum(t.get('sum_of_losing_pnl', 0) for t in token_stats)
        total_duration_seconds = sum(t.get('total_duration_seconds', 0) for t in token_stats)

        # Calculate derived metrics
        win_rate = (total_winning_cycles / total_completed_cycles * 100) if total_completed_cycles > 0 else 0
        average_win_amount = sum_of_winning_pnl / total_winning_cycles if total_winning_cycles > 0 else 0
        average_loss_amount = sum_of_losing_pnl / total_losing_cycles if total_losing_cycles > 0 else 0
        profit_factor = sum_of_winning_pnl / sum_of_losing_pnl if sum_of_losing_pnl > 0 else float('inf') if sum_of_winning_pnl > 0 else 0
        expectancy = (total_realized_pnl / total_completed_cycles) if total_completed_cycles > 0 else 0
        largest_winning_cycle = max((t.get('largest_winning_cycle_pnl', 0) for t in token_stats), default=0)
        largest_losing_cycle = max((t.get('largest_losing_cycle_pnl', 0) for t in token_stats), default=0)
        
        # Average trade duration
        average_trade_duration_seconds = total_duration_seconds / total_completed_cycles if total_completed_cycles > 0 else 0
        average_trade_duration_formatted = self._format_duration(average_trade_duration_seconds)

        # ROI calculation
        initial_balance = float(self.db._get_metadata('initial_balance') or '0.0')
        roi_percentage = (total_realized_pnl / initial_balance * 100) if initial_balance > 0 else 0

        # Calculate best and worst performing tokens
        best_token_name = "N/A"
        best_token_pnl_pct = 0.0
        worst_token_name = "N/A" 
        worst_token_pnl_pct = 0.0
        
        for token in token_stats:
            if token.get('total_completed_cycles', 0) > 0:
                total_pnl = token.get('total_realized_pnl', 0)
                entry_volume = token.get('total_entry_volume', 0)
                
                if entry_volume > 0:
                    pnl_pct = (total_pnl / entry_volume) * 100
                    
                    if best_token_name == "N/A" or pnl_pct > best_token_pnl_pct:
                        best_token_name = token['token']
                        best_token_pnl_pct = pnl_pct
                    
                    if worst_token_name == "N/A" or pnl_pct < worst_token_pnl_pct:
                        worst_token_name = token['token']
                        worst_token_pnl_pct = pnl_pct

        return {
            'total_realized_pnl': total_realized_pnl,
            'total_completed_cycles': total_completed_cycles,
            'total_winning_cycles': total_winning_cycles,
            'total_losing_cycles': total_losing_cycles,
            'total_cancelled_cycles': total_cancelled_cycles,
            'win_rate': win_rate,
            'total_entry_volume': total_entry_volume,
            'total_exit_volume': total_exit_volume,
            'total_trading_volume': total_entry_volume,  # Alias for compatibility
            'sum_of_winning_pnl': sum_of_winning_pnl,
            'sum_of_losing_pnl': sum_of_losing_pnl,
            'average_win_amount': average_win_amount,
            'average_loss_amount': average_loss_amount,
            'avg_win': average_win_amount,  # Alias for compatibility
            'avg_loss': average_loss_amount,  # Alias for compatibility
            'profit_factor': profit_factor,
            'expectancy': expectancy,
            'largest_winning_cycle': largest_winning_cycle,
            'largest_losing_cycle': largest_losing_cycle,
            'largest_win': largest_winning_cycle,  # Alias for compatibility
            'largest_loss': largest_losing_cycle,  # Alias for compatibility
            'total_wins': total_winning_cycles,  # Alias for compatibility
            'total_losses': total_losing_cycles,  # Alias for compatibility
            'roi_percentage': roi_percentage,
            'initial_balance': initial_balance,
            'current_balance': initial_balance + total_realized_pnl,
            'total_duration_seconds': total_duration_seconds,
            'average_trade_duration_seconds': average_trade_duration_seconds,
            'average_trade_duration_formatted': average_trade_duration_formatted,
            'avg_trade_duration': average_trade_duration_formatted,  # Alias for compatibility
            'best_performing_token': {'name': best_token_name, 'pnl_percentage': best_token_pnl_pct},
            'worst_performing_token': {'name': worst_token_name, 'pnl_percentage': worst_token_pnl_pct}
        }

    def get_token_performance(self, limit: int = 20) -> List[Dict[str, Any]]:
        """Get performance stats by token, sorted by total PnL."""
        formatter = get_formatter()
        
        token_stats = self.db._fetch_query(
            "SELECT * FROM token_stats ORDER BY total_realized_pnl DESC LIMIT ?", 
            (limit,)
        )
        
        for token in token_stats:
            total_cycles = token.get('total_completed_cycles', 0)
            winning_cycles = token.get('winning_cycles', 0)
            
            # Calculate win rate
            token['win_rate'] = (winning_cycles / total_cycles * 100) if total_cycles > 0 else 0
            
            # Calculate profit factor
            sum_winning = token.get('sum_of_winning_pnl', 0)
            sum_losing = token.get('sum_of_losing_pnl', 0)
            token['profit_factor'] = sum_winning / sum_losing if sum_losing > 0 else float('inf') if sum_winning > 0 else 0
            
            # Format durations
            total_duration = token.get('total_duration_seconds', 0)
            avg_duration = total_duration / total_cycles if total_cycles > 0 else 0
            token['average_trade_duration_formatted'] = self._format_duration(avg_duration)
            
            # Token display name (use token as-is)
            token['display_name'] = token['token'].upper()
        
        return token_stats

    def get_balance_history(self, days: int = 30) -> Tuple[List[Dict[str, Any]], Dict[str, Any]]:
        """Get balance history for the last N days with detailed statistics."""
        balance_history = self.db._fetch_query(
            "SELECT * FROM balance_history WHERE timestamp >= datetime('now', '-{} days') ORDER BY timestamp ASC".format(days)
        )
        
        if not balance_history:
            return [], {}
        
        # Calculate statistics
        balances = [item['balance'] for item in balance_history]
        
        peak_balance = max(balances)
        current_balance = balances[-1] if balances else 0
        
        # Calculate max drawdown
        running_max = 0
        max_drawdown = 0
        max_drawdown_percentage = 0
        
        for balance in balances:
            if balance > running_max:
                running_max = balance
            
            drawdown = running_max - balance
            drawdown_percentage = (drawdown / running_max * 100) if running_max > 0 else 0
            
            if drawdown > max_drawdown:
                max_drawdown = drawdown
                max_drawdown_percentage = drawdown_percentage
        
        # Calculate period return
        initial_balance_period = balances[0] if balances else 0
        period_pnl = current_balance - initial_balance_period
        period_return_percentage = (period_pnl / initial_balance_period * 100) if initial_balance_period > 0 else 0
        
        stats = {
            'peak_balance': peak_balance,
            'current_balance': current_balance,
            'max_drawdown': max_drawdown,
            'max_drawdown_percentage': max_drawdown_percentage,
            'period_pnl': period_pnl,
            'period_return_percentage': period_return_percentage,
            'data_points': len(balance_history)
        }
        
        return balance_history, stats

    def get_live_max_drawdown(self) -> Tuple[float, float]:
        """Get the current live maximum drawdown from metadata."""
        try:
            # Try to get from DrawdownMonitor first (newer system)
            max_drawdown_pct = float(self.db._get_metadata('drawdown_max_drawdown_pct') or '0.0')
            peak_balance = float(self.db._get_metadata('drawdown_peak_balance') or '0.0')
            
            if max_drawdown_pct > 0:
                max_drawdown_absolute = peak_balance * (max_drawdown_pct / 100)
                return max_drawdown_absolute, max_drawdown_pct
            
            # Fallback to legacy keys if DrawdownMonitor data not available
            max_drawdown_live = float(self.db._get_metadata('max_drawdown_live') or '0.0')
            max_drawdown_live_percentage = float(self.db._get_metadata('max_drawdown_live_percentage') or '0.0')
            return max_drawdown_live, max_drawdown_live_percentage
        except (ValueError, TypeError):
            return 0.0, 0.0

    def update_live_max_drawdown(self, current_balance: float) -> bool:
        """Update live maximum drawdown tracking."""
        try:
            # Get peak balance
            peak_balance = float(self.db._get_metadata('peak_balance') or '0.0')
            
            # Update peak if current balance is higher
            if current_balance > peak_balance:
                peak_balance = current_balance
                self.db._set_metadata('peak_balance', str(peak_balance))
            
            # Calculate current drawdown
            current_drawdown = peak_balance - current_balance
            current_drawdown_percentage = (current_drawdown / peak_balance * 100) if peak_balance > 0 else 0
            
            # Get current max drawdown
            max_drawdown_live = float(self.db._get_metadata('max_drawdown_live') or '0.0')
            max_drawdown_live_percentage = float(self.db._get_metadata('max_drawdown_live_percentage') or '0.0')
            
            # Update max drawdown if current is worse
            if current_drawdown > max_drawdown_live:
                self.db._set_metadata('max_drawdown_live', str(current_drawdown))
                self.db._set_metadata('max_drawdown_live_percentage', str(current_drawdown_percentage))
                logger.info(f"📉 New max drawdown: ${current_drawdown:.2f} ({current_drawdown_percentage:.2f}%)")
                return True
            
            return False
            
        except Exception as e:
            logger.error(f"❌ Error updating live max drawdown: {e}")
            return False

    def calculate_sharpe_ratio(self, days: int = 30) -> Optional[float]:
        """Calculate Sharpe ratio from balance history."""
        try:
            balance_history = self.db._fetch_query(
                "SELECT balance, timestamp FROM balance_history WHERE timestamp >= datetime('now', '-{} days') ORDER BY timestamp ASC".format(days)
            )
            
            if len(balance_history) < 2:
                return None
            
            # Calculate daily returns
            daily_returns = []
            for i in range(1, len(balance_history)):
                prev_balance = balance_history[i-1]['balance']
                curr_balance = balance_history[i]['balance']
                daily_return = (curr_balance - prev_balance) / prev_balance if prev_balance > 0 else 0
                daily_returns.append(daily_return)
            
            if not daily_returns:
                return None
            
            # Calculate Sharpe ratio (assuming 0% risk-free rate)
            mean_return = np.mean(daily_returns)
            std_return = np.std(daily_returns, ddof=1) if len(daily_returns) > 1 else 0
            
            if std_return == 0:
                return None
            
            # Annualized Sharpe ratio (approximately)
            sharpe_ratio = (mean_return / std_return) * math.sqrt(365) if std_return > 0 else 0
            return sharpe_ratio
            
        except Exception as e:
            logger.error(f"❌ Error calculating Sharpe ratio: {e}")
            return None

    def calculate_max_consecutive_losses(self) -> int:
        """Calculate maximum consecutive losing trades."""
        try:
            # Get all completed trades ordered by date
            completed_trades = self.db._fetch_query("""
                SELECT realized_pnl FROM trades 
                WHERE status = 'position_closed' AND realized_pnl IS NOT NULL
                ORDER BY timestamp ASC
            """)
            
            if not completed_trades:
                return 0
            
            max_consecutive = 0
            current_consecutive = 0
            
            for trade in completed_trades:
                pnl = trade.get('realized_pnl', 0)
                if pnl < 0:  # Losing trade
                    current_consecutive += 1
                    max_consecutive = max(max_consecutive, current_consecutive)
                else:  # Winning trade or breakeven
                    current_consecutive = 0
            
            return max_consecutive
            
        except Exception as e:
            logger.error(f"❌ Error calculating max consecutive losses: {e}")
            return 0

    def get_risk_metrics(self) -> Dict[str, Any]:
        """Calculate various risk metrics."""
        try:
            # Get performance stats
            perf_stats = self.get_performance_stats()
            
            # Get live max drawdown
            max_drawdown_live, max_drawdown_live_percentage = self.get_live_max_drawdown()
            
            # Calculate Sharpe ratio
            sharpe_ratio = self.calculate_sharpe_ratio()
            
            # Calculate max consecutive losses
            max_consecutive_losses = self.calculate_max_consecutive_losses()
            
            # Calculate Calmar ratio (annual return / max drawdown)
            annual_return_percentage = perf_stats.get('roi_percentage', 0)  # This is total ROI, approximate as annual
            calmar_ratio = annual_return_percentage / max_drawdown_live_percentage if max_drawdown_live_percentage > 0 else None
            
            # Calculate risk/reward ratio
            avg_win = perf_stats.get('average_win_amount', 0)
            avg_loss = perf_stats.get('average_loss_amount', 0)
            risk_reward_ratio = avg_win / avg_loss if avg_loss > 0 else None
            
            return {
                'max_drawdown_live': max_drawdown_live,
                'max_drawdown_live_percentage': max_drawdown_live_percentage,
                'sharpe_ratio': sharpe_ratio,
                'max_consecutive_losses': max_consecutive_losses,
                'calmar_ratio': calmar_ratio,
                'risk_reward_ratio': risk_reward_ratio,
                'profit_factor': perf_stats.get('profit_factor', 0),
                'win_rate': perf_stats.get('win_rate', 0)
            }
            
        except Exception as e:
            logger.error(f"❌ Error calculating risk metrics: {e}")
            return {}

    def get_period_performance(self, start_date: str, end_date: str) -> Dict[str, Any]:
        """Get performance statistics for a specific date range."""
        try:
            # Get daily stats for the period
            daily_stats = self.db._fetch_query("""
                SELECT date, SUM(realized_pnl) as pnl, SUM(completed_cycles) as trades, 
                       SUM(exit_volume) as volume
                FROM daily_aggregated_stats 
                WHERE date BETWEEN ? AND ?
                GROUP BY date
                ORDER BY date ASC
            """, (start_date, end_date))
            
            if not daily_stats:
                return {
                    'period_start': start_date,
                    'period_end': end_date,
                    'total_pnl': 0,
                    'total_trades': 0,
                    'total_volume': 0,
                    'win_rate': 0,
                    'trading_days': 0,
                    'average_daily_pnl': 0
                }
            
            total_pnl = sum(day.get('pnl', 0) or 0 for day in daily_stats)
            total_trades = sum(day.get('trades', 0) or 0 for day in daily_stats)
            total_volume = sum(day.get('volume', 0) or 0 for day in daily_stats)
            trading_days = len([day for day in daily_stats if (day.get('trades', 0) or 0) > 0])
            
            average_daily_pnl = total_pnl / trading_days if trading_days > 0 else 0
            
            return {
                'period_start': start_date,
                'period_end': end_date,
                'total_pnl': total_pnl,
                'total_trades': total_trades,
                'total_volume': total_volume,
                'trading_days': trading_days,
                'average_daily_pnl': average_daily_pnl,
                'daily_stats': daily_stats
            }
            
        except Exception as e:
            logger.error(f"❌ Error calculating period performance: {e}")
            return {}

    def get_recent_performance_trend(self, days: int = 7) -> Dict[str, Any]:
        """Get recent performance trend analysis."""
        try:
            end_date = datetime.now(timezone.utc).date()
            start_date = end_date - timedelta(days=days)
            
            period_stats = self.get_period_performance(
                start_date.strftime('%Y-%m-%d'), 
                end_date.strftime('%Y-%m-%d')
            )
            
            # Calculate trend direction
            daily_pnls = [day.get('pnl', 0) or 0 for day in period_stats.get('daily_stats', [])]
            
            if len(daily_pnls) >= 2:
                # Simple linear trend
                x = list(range(len(daily_pnls)))
                slope = np.polyfit(x, daily_pnls, 1)[0] if len(daily_pnls) > 1 else 0
                trend_direction = 'up' if slope > 0 else 'down' if slope < 0 else 'flat'
            else:
                trend_direction = 'insufficient_data'
                slope = 0
            
            return {
                'days': days,
                'trend_direction': trend_direction,
                'slope': slope,
                **period_stats
            }
            
        except Exception as e:
            logger.error(f"❌ Error calculating recent performance trend: {e}")
            return {'days': days, 'trend_direction': 'error'} 